package org.objectweb.jonathan.libs.kernel.decoders;

import junit.framework.*;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Element;
import org.objectweb.jonathan.apis.kernel.MultiFactory;
import org.objectweb.jonathan.apis.kernel.ClassPathFinder;
import org.objectweb.jonathan.apis.kernel.decoders.File2Configuration;

import org.objectweb.jonathan.libs.kernel.JConfigurationFactory;

import org.objectweb.jonathan.libs.kernel.decoders.JAtom;

import java.util.Enumeration;
import java.util.Properties;


public class SequenceTest extends TestCase {

   Context theContext = null;

   public SequenceTest(String name) {
      super(name);
      theContext = this.getContext();
   }

	public static void main(String args[]) {
		junit.textui.TestRunner.run(SequenceTest.class);
	}   

   protected void setUp() {

   }
   
   public void testObjectSequence() {
      try {         
         Object $object_seq = theContext.getValue("Object_sequence",(char) 0);
         assert("$object_seq is not an Object[]", $object_seq instanceof
                Object[]);
         Object[] $object_seq_1 = (Object[])$object_seq;
         Object $anObject = (Object)$object_seq_1[0];
         
         String $aString = (String)$object_seq_1[1];
         assert("String is not empty", "".equals($aString));
         ArithmeticException $anException = (ArithmeticException)$object_seq_1[2];
         assert("Exception has wrong message", "no_value".equals($anException.getMessage()));
         $aString = (String)$object_seq_1[3];
         assert("String test is missing", "test".equals($aString));
         Integer $anInteger = (Integer)$object_seq_1[4];
         assert("Integer has wrong value", ($anInteger.intValue() == 4));
         
         assert("Sequence has wrong size", ($object_seq_1.length == 5));

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      }catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      }
   }   

   public void testStringSequence() {
      try {         
         Object $string_seq = theContext.getValue("String_sequence",(char) 0);
         assert("$string_seq is not an String[] but a "+$string_seq.getClass(),
                $string_seq instanceof String[]);

         String[] $the_seq = (String[])$string_seq;
         String $string0 = $the_seq[0];
         assert("wrong string[0]", "test0".equals($string0));

         String $string1 = $the_seq[1];
         assert("wrong string[1]", "test1".equals($string1));

         assert("Sequence has wrong size", ($the_seq.length == 2));

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      }catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      }
   }
   
   public void testIntSequence() {
      try {         
         Object $int_seq = theContext.getValue("int_sequence",(char) 0);
         assert("$int_seq is not an int[] but a "+$int_seq.getClass(), $int_seq instanceof int[]);

         int [] $the_seq = (int[])$int_seq;

         assert("Sequence has wrong size", ($the_seq.length == 3));

         for (int $count=0;$count<3;$count++) {
            int $each = $the_seq[$count];
            assert("wrong element value at index = "+$count,$each == $count);
         }

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      }
   }
   public void testCharSequence() {
      try {         
         Object $char_seq = theContext.getValue("char_sequence",(char) 0);
         assert("$char_seq is not an char[] but a "+$char_seq.getClass(), $char_seq instanceof char[]);

         char [] $the_seq = (char[])$char_seq;

         assert("Sequence has wrong size", ($the_seq.length == 2));

         char $each = $the_seq[0];
         assert("wrong element value at 0 ",$each == 'h');
         $each = $the_seq[1];
         assert("wrong element value at 1",$each == 'o');

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      }
   }

   public void testFloatSequence() {
      try {         
         Object $float_seq = theContext.getValue("float_sequence",(char) 0);
         assert("$float_seq is not an float[] but a "+$float_seq.getClass(), $float_seq instanceof float[]);

         float [] $the_seq = (float[])$float_seq;

         assert("Sequence has wrong size", ($the_seq.length == 2));

         float $each = $the_seq[0];
         assert("wrong element value",$each == 3.14159f);
         $each = $the_seq[1];
         assert("wrong element value",$each == 31.4159f);

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      }
   }

   public void testDoubleSequence() {
      try {         
         Object $double_seq = theContext.getValue("double_sequence",(char) 0);
         assert("$double_seq is not an double[] but a "+$double_seq.getClass(), $double_seq instanceof double[]);

         double [] $the_seq = (double[])$double_seq;

         assert("Sequence has wrong size", ($the_seq.length == 2));

         double $each = $the_seq[0];
         assert("wrong element value",$each == 3.14159d);
         $each = $the_seq[1];
         assert("wrong element value",$each == 31.4159d);

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      }
   }

   public void testAssemblage() {
     try {         
         Object $ass = theContext.getValue("assemblage",(char) 0);
         assert("$ass is not a String but a "+$ass.getClass(), $ass instanceof String);

         String $the_string = (String)$ass;

         assert("the string is not well initialized (" + $the_string +")", "hello".equals($the_string));

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      } 
   }

   public void testAlias() {
     try {         
         Object $ass = theContext.getValue("alias_test/new_sequence",'/');
         assert("$ass is not a String[] but a "+$ass.getClass(), $ass instanceof String[]);

         String[] $the_array = (String[])$ass;

         assert("the string array is not well initialized ", "test0".equals($the_array[0]));
         assert("the string array is not well initialized because of alias", "test1".equals($the_array[1]));

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      } 
   }

   /**
    * This test shows a potential bug, if sequences cannot be traversed by
    * aliases. The sequence contains an assemblage, which uses an alias
    * in its configuration.
    */
   public void testInternalAlias() {
     try {         
         Object $ass = theContext.getValue("internal_alias_test/sequence",'/');
         assert("string[] could not be built because instanciation did not work", $ass instanceof String[]);
         String[] $the_array = (String[])$ass;

         assert("alias has not worked in the assemblage", "test".equals($the_array[0]));
         assert("alias has not worked between the assemblages", "test".equals($the_array[1]));
         assert("alias has not worked for factory between the assemblages", "test".equals($the_array[2]));

         String $alias1 =
            (String)theContext.getValue("internal_alias_test/external_alias1",'/');
         assert("outside direct alias in sequence failed", "test".equals($alias1));
         String $alias2 = (String)theContext.getValue("internal_alias_test/external_alias2",'/');
         assert("outside indirect alias in sequence failed", "test".equals($alias2));
         String $alias3 = (String)theContext.getValue("internal_alias_test/external_alias3",'/');
         assert("outside alias to sequence element 0 failed", "test".equals($alias3));
         String $alias_vicieux = (String)theContext.getValue("internal_alias_test/alias_vicieux",'/');
         assert("Vicious test failed", "test vicieux".equals($alias_vicieux));

         String $direct_access =
            (String)theContext.getValue("internal_alias_test/sequence/1",'/');
         assert("direct access to element failed","test".equals($direct_access));

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      } 
   }

   public void testNoInitAlias() {
     try {         
        // we ask for element 0 without asking for the sequence first
        //         Object $ass0 = theContext.getValue("noinit_alias_test/never_ask_me",'/');
        
        
         Object $direct = theContext.getValue("noinit_alias_test/never_ask_me/0",'/');
         assert("Direct access to sequence element does not work", $direct instanceof String);
         Object $aliased = theContext.getValue("noinit_alias_test/ask_me",'/');
         assert("Aliased access to sequence element does not work", $aliased instanceof String);
         assert("Not the right contents", "you've got me".equals($aliased));

      } catch (ClassCastException ccex) {
         ccex.printStackTrace();
         fail("Elements typing error");
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      } 
   }

//    public void testLazySequence() {
//       LazySequence seq = (LazySequence) theContext.getValue("lazy sequence",
//                                                             (char) 0);
//       assertSame(seq,seq.getElem());
//    }
   

   public Context getContext() {
      try {
         
         Context $initial_context = BootKernel.newConfiguration(SequenceTest.class);
         ClassPathFinder $finder = (ClassPathFinder) $initial_context.getValue("classpath_finder",(char) 0);
         File2Configuration $decoder = (File2Configuration) 
            $initial_context.getValue("jonathan/decoders/OpenXMLDecoder/instance",'/');
      
         JConfigurationFactory $context_factory = new JConfigurationFactory();
      
         Context $context = $context_factory.newContext(null,$finder);
      
         $decoder.importElement($context,
                                "org/objectweb/jonathan/libs/kernel/decoders/sequence.kcf",
                                $context_factory,
                                $finder);
         return $context;
      } catch (Exception e) {
         e.printStackTrace();
         return null;
      }
   }
}




