/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Priscille Durville
 *
 * 
 */

package org.objectweb.jonathan.apis.kernel;

/** 
 * <code>Instanciable</code> represents an object (a service) created by its factory.
 * <p>
 * Instanciables are used in order to prevent side effects while creating services.
 * For the time being, they are used in case of circularity not to call their
 * factory twice by stack returned when the service still exists.
 */
public interface Instanciable {
   
   /** 
    * Test if the service still exists.
    * @return true if the service still exists, false otherwise.
    */
   public boolean isInstanciated();

   /** 
    * Return the service's instance.
    * @return the service's instance.
    */
   public Object getInstance();

}
