/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 * 
 */

package org.objectweb.jonathan.libs.kernel;

import org.objectweb.jonathan.apis.kernel.Element;
import org.objectweb.jonathan.apis.kernel.Name;
import org.objectweb.jonathan.apis.kernel.Component;

import org.objectweb.jonathan.libs.helpers.NameHelpers;

/**
 * Basic implementation of a runtime element. Its behaviour
 * is a property component's. Must be subtyped to behave
 * differently.
 */
public class JTreeElement extends JElement implements TreeElement {

   JTreeElement() {
      super();
   }
   
   public JTreeElement(Component _container, String _name,Component _target) {
      super(_container,_name,_target);
   }
   
   public Name getAbsoluteName() {
      return getAbsoluteName(null);
   }

   public Name getAbsoluteName(Name subname) {
      Name new_name = NameHelpers.newName(getName(),subname);
      if (container != null) {
         Element reference = container.getReference();
         if (reference == null) {
            return NameHelpers.newName("",new_name);
         } else if (reference instanceof TreeElement) {
            return ((TreeElement) reference).getAbsoluteName(new_name);
         } else {
            return NameHelpers.newName("<undefined>",new_name);
         }
      } else {
         return NameHelpers.newName("<undefined>",new_name);
      }
   }            
      
   public String toString() {
      return "Element[" + getAbsoluteName() + " " + component + "]";
   }
}
