/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 *
 * -------------------------------------------------------------------
 * $Id: JAssemblage.java,v 1.1.1.1 2002/08/06 13:13:52 bmi Exp $
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Element;
import org.objectweb.jonathan.apis.kernel.Factory;
import org.objectweb.jonathan.apis.kernel.Initializer;
import org.objectweb.jonathan.apis.kernel.Instanciable;
import org.objectweb.jonathan.apis.kernel.JonathanException;
import org.objectweb.jonathan.apis.kernel.MultiFactory;
import org.objectweb.jonathan.apis.kernel.Name;

import org.objectweb.jonathan.libs.helpers.TraceHelpers;
import org.objectweb.jonathan.libs.helpers.NameHelpers;
import org.objectweb.jonathan.libs.kernel.JTreeElement;
import org.objectweb.jonathan.libs.kernel.Forkable;
import org.objectweb.jonathan.libs.helpers.HTable;
import org.objectweb.jonathan.libs.kernel.Forkable;
import org.objectweb.jonathan.apis.kernel.MultiFactory;

import java.lang.reflect.Method;
import java.util.List;
import java.util.LinkedList;
import java.util.ListIterator;



/**
 * Runtime assemblage component. Initialisation issues
 * are dealt with by the {@link LazyComponent} implementation.
 * Provides the basic behaviour of a
 * <code>MultiFactory</code> too: holds a set of implementation alternatives.
 */
public class JAssemblage extends LazyComponent implements Instanciable, MultiFactory, Initializer {

    // following code from KAssemblage
    HTable alternatives = null;
    List alternatives_list;

   static final Name SELF_NAME = NameHelpers.newName(".",(char) 0);
   static final Name ROOT_NAME = NameHelpers.newName("",(char) 0);
   static final String FACTORY_NAME = "Factory";
   static final String CONFIGURATION_NAME = "Configuration";
   
   protected Component factory;
   protected Component configuration;

   protected Element factory_element;
   protected Element configuration_element;

   protected Element self;

   protected int alternative = -1;
        
   /**
    * creates a new runtime assemblage component.
    */
//    public JAssemblage(Component _factory,Component _configuration) {
//       factory = _factory;
//       configuration = _configuration;
//       factory_element = new JTreeElement(this,FACTORY_NAME,factory);
//       factory.setReference(factory_element);
//       configuration_element = new JTreeElement(this,CONFIGURATION_NAME,configuration);
//       configuration.setReference(configuration_element);
//       self = new JTreeElement(this,".",this);

//    }
      
//    public JAssemblage(Component _factory,Component _configuration,int _alternative) {
//        this(_factory,_configuration);

//       alternative = _alternative;
//    }

    /**
    * Creates a new assemblage whose alternatives
    * are listed in <B>_alternatives</B> as JAlternatives.
    */
    public JAssemblage(Component _factory, Component _configuration, List _alternatives) {
// 	this(_factory,_configuration);

      factory = _factory;
      configuration = _configuration;
      factory_element = new JTreeElement(this,FACTORY_NAME,factory);
      factory.setReference(factory_element);
      configuration_element = new JTreeElement(this,CONFIGURATION_NAME,configuration);
      configuration.setReference(configuration_element);
      self = new JTreeElement(this,".",this);

	alternatives_list = _alternatives;
	alternatives = HTable.newTable();
	JAlternative $alternative = null;
	for (ListIterator $iterator = _alternatives.listIterator(); $iterator.hasNext();) {
	    $alternative = (JAlternative) $iterator.next();
	    alternatives.put($alternative.id,$alternative);
	}
    }

    /**
     * Creates a new assemblage whose alternatives
     * are listed in <B>_alternatives</B> as JAlternatives.
     */
    public JAssemblage(Component _factory,Component _configuration,int _alt,List _alternatives) {
// 	this(_factory,_configuration,_alt);

	this(_factory, _configuration, _alternatives);
	alternative = _alt;

    }

    public JAssemblage(Component _factory, Component _configuration) {
	this(_factory,_configuration,new LinkedList());
    }


    public JAssemblage(Component _factory,Component _configuration,int _alt){
	this(_factory,_configuration,_alt,new LinkedList());
    }

//    public Component duplicate() {
//       return new JAssemblage(((Forkable)factory).internalFork(),
//                              ((Forkable) configuration).internalFork(),
//                              alternative);
//    }
  
   public Component duplicate() {
      return new JAssemblage(((Forkable)factory).internalFork(),((Forkable)configuration).internalFork(),alternative,alternatives_list);
   }

   public Element getElement(Name _name) {
      String $id  = _name.getId();
      Name $subname = _name.getSubName();
      Element $local = getLocalElement($id);
      if ($local == null || $subname == null) {
         return $local;
      } else {
         Component $local_component = $local.getComponent();
         return $local_component != null ? $local_component.getElement($subname) : null;
      }
   }

   Element getLocalElement(String _id) {
      //System.err.println("Searching \"" + _id + "\" in " + this);
      if (FACTORY_NAME.equals(_id)) {
         return factory_element;
      } else if (CONFIGURATION_NAME.equals(_id)) {
         return configuration_element;
      } else if (".".equals(_id)) {
         return self; 
      } else if ("..".equals(_id)) {
         return reference != null ? reference.getContainer().getElement(SELF_NAME) : null;
      } else if ("".equals(_id)) {
         if (reference != null) {
            return reference.getContainer().getElement(ROOT_NAME);
         } else {
            return self;
         }
      } else {
         return null;
      }
   }
   
   
//    public String toString() {
//       if (! initialized) {
//          return "JAssemblage[" + factory + " " + configuration + " " + alternative + "]";
//       } else {
//          return "JAssemblage[" + type + " " + value + " " + 
//             factory + " " + configuration + " " + alternative + "]";
//       }            
//    }

   public String toString() {
      return "JAssemblage[" + factory + " " + configuration +
	  " " + alternative + " " + alternatives + "]";
   }


   /** 
    * Return a wrapper of this JComponent value for factory aspects.
    * @return a wrapper of this JComponent value for factory aspects.
    * @exception JonathanException
    */
   public Object getFactoryValue() throws JonathanException{
      return this;
   }

    
   /**
    * Implementation of the MultiFactory interface's newObject method.
    * Returns a new instance of the atom's implementation
    * alternative whose ID is <code>_alternative</code>.
    * @param _c where the instanciation parameters are found
    * @param _alternative the alternative to instantiate
    * @param _instance the method's caller.
    * @return an new <code>_alternative</code>'s instance
    * @exception JonathanException if something goes wrong.
    */
   public Object newObject(Context _c,int _alternative,Instanciable _instance) throws JonathanException {
      JAlternative $alternative = (JAlternative) alternatives.get(_alternative);
      if ($alternative == null) {
         throw new JonathanException("ERROR in factory [" +
                                     reference +
                                     "] : alternative " + _alternative +
                                     " not found.");
      }
      Method $constructor = $alternative.mconstructor;
      if ($constructor == null) {
         Component $context = reference.getContainer();
         $constructor = DecoderHelper.getMethod(this.getValue().getClass(),
                                                $alternative.construction_name,
                                                $alternative.arguments,
                                                $context);
         $alternative.mconstructor = $constructor;
      }
      return DecoderHelper.apply(this.getValue(),$constructor,$alternative.arguments,_c,_instance);
   }
   

   /**
    * Implementation of the Initializer interface's initialize method. 
    * Initializes an instance of the atom's implementation
    * alternative whose ID is <code>_alternative</code> by calling its
    * setters.
    * @param _obj the object to initialize
    * @param _c where the initialisation parameters are found
    * @param _alternative the alternative to instantiate
    * @exception JonathanException if something goes wrong.
    */
   public void initialize(Object _obj,Context _c,int _alternative) throws JonathanException {
      if (_obj == null) return;
      JAlternative $alternative = (JAlternative) alternatives.get(_alternative);
      if ($alternative == null) {
         throw new JonathanException("ERROR in factory [" +
                                     reference +
                                     "] : alternative " + _alternative +
                                     " not found.");
      }
      Component $context = reference.getContainer(); 
      List $meths = $alternative.setters;
      for (int i=0; i<$meths.size(); i++){
         JSetter $current = (JSetter)$meths.get(i);
         Method $meth = DecoderHelper.getMethod(_obj.getClass(),$current.name,$current.args,$context);
         Object $returnedByMeth = DecoderHelper.apply(_obj,$meth,$current.args,_c,null);
      }
   }


   /**
    * JAssemblage specific initialization code. Seeks & instantiates
    * its factory, and then asks its factory to instantiate the
    * assemblage's own value. Used internally by {@link LazyComponent}.
    */
   protected final void initialize() {
      if (initialized){return;}
      try {
         Factory $factory = null;
         MultiFactory $multi_factory = null;
         Initializer $initializer = null;
         Context $configuration = null;

         Object $component = factory.getValue();
         if ($component instanceof MultiFactory && alternative >= 0) {//IF
            $multi_factory = (MultiFactory) $component;
            if ($component instanceof Initializer) {
               $initializer=(Initializer)$component;
            }
         } else if ($component instanceof Factory) {//EF
            $factory = (Factory) $component;
         } else if ($component instanceof MultiFactory) {
            System.err.println("WARNING in assemblage [" + reference + 
                               "] : No valid alternative number provided.");
            return;
         } else {//Atom or Assemblage
            try {
               $component = factory.getFactoryValue();
               if ($component instanceof MultiFactory){
                  $multi_factory = (MultiFactory) $component;
                  if ($component instanceof Initializer) {
                     $initializer=(Initializer)$component;
                  }
               }
            }catch (JonathanException je){
               System.err.println("WARNING in assemblage [" +
                               reference +
                               "] : Could not instanciate " +
                               factory_element +
                               " as a factory.");
               return;
            }
         }
                
         $component = configuration.getValue();
         if ($component instanceof Context) {
            $configuration = (Context) $component;
         } else {
            System.err.println("WARNING in assemblage [" +
                               reference + 
                               "] : Could not instanciate  " +
                               configuration_element + " as a context.");
            return;
         }
            
         // We now have an instance of the factory & context, let's try to build an 
         // instance of the service and, then, to initialize this instance.
         try{
            if (value != Context.NO_VALUE){
               return;//still exists.
            }
            if ($factory != null) {
               value = $factory.newObject($configuration);
            } else {
               Object $new_value = $multi_factory.newObject($configuration,alternative,this);
               if (value == Context.NO_VALUE){
                  //System.err.println("Computed value for "  + reference + ": " + $new_value);
                  value = $new_value;
                  if ($initializer != null) {
                     $initializer.initialize(value,$configuration,alternative);
                  }
               }
            }
         } catch (JonathanException e){
            System.err.println("WARNING in assemblage [" +
                               reference + 
                               "] : Could not create an instance of service from factory " +
                               factory_element + 
                               ": " + TraceHelpers.convertStackToString(e));
            value = Context.NO_VALUE;
            return;
         }
            
         if (value != null){
            type = value.getClass();
         }
         return;
      } finally {
         if (! initialized) initialized = true;
      }
   }

   /** 
    * From Instanciable interface.
    * @return true if the value exists, false otherwise.
    */
   public boolean isInstanciated(){
      if (value == Context.NO_VALUE){
         return false;
      } else {
         return true;
      }
   }

   /** 
    * From Instanciable interface.
    * @return the value of the component (without computing it!).
    */
   public Object getInstance(){
      return value;
   }
   
}
