/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 *
 * -------------------------------------------------------------------
 * $Id: JAtom.java,v 1.1.1.1 2002/08/06 13:13:52 bmi Exp $
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.JonathanException;
import org.objectweb.jonathan.apis.kernel.ClassPathFinder;

import org.objectweb.jonathan.libs.helpers.TraceHelpers;
import org.objectweb.jonathan.libs.helpers.NameHelpers;
import org.objectweb.jonathan.libs.kernel.ClassPathFinderImpl;
import org.objectweb.jonathan.libs.helpers.HTable;
import org.objectweb.jonathan.apis.kernel.MultiFactory;
import org.objectweb.jonathan.apis.kernel.Initializer;
import org.objectweb.jonathan.apis.kernel.Instanciable;

import java.lang.reflect.Method;
import java.util.List;
import java.util.LinkedList;
import java.util.ListIterator;



/**
 * Runtime atom component. Initialisation issues
 * are dealt with by the {@link LazyComponent} implementation. 
 * Provides the basic behaviour of a
 * <code>MultiFactory</code> too: holds a set of implementation alternatives.
 */
public class JAtom extends LazyComponent implements MultiFactory, Initializer {

    // code from KAtom follows
    HTable alternatives = null;
    List alternatives_list;
    // end

   protected String classe;

//    public JAtom(String _class) {
//       super();
//       classe = _class;
//    }

   /**
    * Creates a new class atom whose alternatives
    * are listed in <B>_alternatives</B> as JAlternatives.
    */
   public JAtom(String _class, List _alternatives) {
//       super(_class);
       super();
       classe = _class;

      alternatives_list = _alternatives;
      alternatives = HTable.newTable();
      JAlternative $alternative = null;
      for (ListIterator $iterator = _alternatives.listIterator(); $iterator.hasNext();) {
         $alternative = (JAlternative) $iterator.next();
         alternatives.put($alternative.id,$alternative);
      }
   }

   /**
    * Creates a new class atom whose alternatives
    * are listed in <B>_alternatives</B> as JAlternatives.
    */
   public JAtom(String _class) {
      this(_class, new LinkedList());
   }

//    public Component duplicate() {
//       return new JAtom(classe);
//    }

   public Component duplicate() {
      return new JAtom(classe, alternatives_list);
   }

//    public String toString() {
//       if (! initialized) {
//          return "JAtom[" + classe + "]";
//       } else {
//          return "JAtom[" + type + " " + value + "]";
//       }            
//    }

   public String toString() {
      return "JAtom[" + classe + " " + alternatives + "]";        
   }

   /** 
    * Return a wrapper of this JComponent value for factory aspects.
    * @return a wrapper of this JComponent value for factory aspects.
    * @exception JonathanException
    */
   public Object getFactoryValue() throws JonathanException{
      return this;
   }


   /**
    * Implementation of the MultiFactory interface's newObject method.
    * Returns a new instance of the atom's implementation
    * alternative whose ID is <code>_alternative</code>.
    * @param _c where the instanciation parameters are found
    * @param _alternative the alternative to instantiate
    * @param _instance the method's caller.
    * @return an new <code>_alternative</code>'s instance
    * @exception JonathanException if something goes wrong.
    */
   public Object newObject(Context _c,int _alternative,Instanciable _instance) throws JonathanException {
      JAlternative $alternative = (JAlternative) alternatives.get(_alternative);
      if ($alternative == null) {
         throw new JonathanException("ERROR in factory [" +
                                     reference +
                                     "] : alternative " + _alternative +
                                     " not found.");
      }
      Method $constructor = $alternative.mconstructor;
      if ($constructor == null) {
         Component $context = reference.getContainer();
         $constructor = DecoderHelper.getMethod(this.getValue().getClass(),
                                                $alternative.construction_name,
                                                $alternative.arguments,
                                                $context);
         $alternative.mconstructor = $constructor;
      }
      return DecoderHelper.apply(this.getValue(),$constructor,$alternative.arguments,_c,_instance);
   }
   
   /**
    * Implementation of the Initializer interface's initialize method. 
    * Initializes an instance of the atom's implementation
    * alternative whose ID is <code>_alternative</code> by calling its
    * setters.
    * @param _obj the object to initialize
    * @param _c where the initialisation parameters are found
    * @param _alternative the alternative to instantiate
    * @exception JonathanException if something goes wrong.
    */
   public void initialize(Object _obj,Context _c,int _alternative) throws JonathanException {
      if (_obj == null) return;
      JAlternative $alternative = (JAlternative) alternatives.get(_alternative);
      if ($alternative == null) {
         throw new JonathanException("ERROR in factory [" +
                                     reference +
                                     "] : alternative " + _alternative +
                                     " not found.");
      }
      Component $context = reference.getContainer(); 
      List $meths = $alternative.setters;
      for (int i=0; i<$meths.size(); i++){
         JSetter $current = (JSetter)$meths.get(i);
         Method $meth = DecoderHelper.getMethod(_obj.getClass(),$current.name,$current.args,$context);
         Object $returnedByMeth = DecoderHelper.apply(_obj,$meth,$current.args,_c,null);
      }
   }

   /**
    * JAtom specific initialization code. Instantiates the JAtom's
    * own value. Used internally by {@link LazyComponent}.
    */
   final protected void initialize() {
      if (! initialized) {
         Component context = reference.getContainer();
         initialized = true;
            
         Object $component = context.getValue(NameHelpers.newName("/classpath_finder",'/'));
         ClassPathFinder $class_finder = null;
            
         if ($component instanceof ClassPathFinder) {
            $class_finder = (ClassPathFinder) $component;
            type = $class_finder.findClass(classe);
         } else {
            type = ClassPathFinderImpl.loadSystemClass(classe);
         }
            
         if (type == null) {
            System.err.println("Warning : could not instantiate atom [" +
                               reference +
                               "] of class " + classe );
            return;
         }
            
         try{
            value = type.newInstance();
            return;
         } catch (IllegalAccessException e){
            System.err.println("Warning : could not instantiate atom [" +
                               reference +
                               "] of type " +
                               type + ": " + TraceHelpers.convertStackToString(e));
            return;
         } catch (InstantiationException e1){
            System.err.println("Warning : could not instantiate atom [" +
                               reference +
                               "] of type "+
                               type + ": " + TraceHelpers.convertStackToString(e1));
            return;
         } catch (ExceptionInInitializerError e2){
            System.err.println("Warning : could not instantiate atom [" +
                               reference +
                               "] of type "+
                               type + ": " + TraceHelpers.convertStackToString(e2));
            return;
         } catch (SecurityException e3){
            System.err.println("Warning : could not instantiate atom [" +
                               reference +
                               "] of type "+
                               type + ": " + TraceHelpers.convertStackToString(e3));
            return;
         }
      }
   }
}
