/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Priscille Durville
 *
 * -------------------------------------------------------------------
 * $ID$
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.JonathanException;

import org.objectweb.jonathan.libs.kernel.decoders.XMLElementDecoder;
import org.objectweb.jonathan.libs.kernel.decoders.XMLElementDecoderDispatcher;

import nanoxml.XMLElement;

import java.util.Enumeration;
import java.util.List;
import java.util.LinkedList;

/**
 * Atom specific XML to runtime component decoder.
 * Used by the {@link OpenXMLDecoder} as one of the built-in
 * component decoders provided it provides to its decoding
 * dispatcher.
 */
public class XMLAtomDecoder implements XMLElementDecoder {

    /**
    * Converts a XML <code>ATOM</code> element to its
    * corresponding atom runtime component.
    * @param _xml_element <code>ATOM</code> tag to be decoded
    * @param _element_name the atom's element name
    * @param _configuration where the atom will be created
    * @param return the corresponding decoded runtime atom
    * component
    */
   public Component decode(XMLElement _xml_element, String _element_name, Context _configuration,
                           XMLElementDecoderDispatcher _dispatcher) 
      throws JonathanException {
      
      String $current_tag = _xml_element.getName();

      if ($current_tag.equals("ATOM")){

         String $class = _xml_element.getStringAttribute("CLASS");
         
         Enumeration $alternatives = _xml_element.enumerateChildren();
         List $alternatives_list = new LinkedList();

         while ($alternatives.hasMoreElements()) {
            XMLElement $alternative = (XMLElement) $alternatives.nextElement();
            $current_tag = $alternative.getName();
            if ($current_tag.equals("ALTERNATIVE")) {
               List $arguments_list = new LinkedList();
               List $setters_list = new LinkedList();
               Enumeration $arguments = $alternative.enumerateChildren();
               while ($arguments.hasMoreElements()) {
                  XMLElement $argument = (XMLElement) $arguments.nextElement();
                  $current_tag = $argument.getName();
                  if ($current_tag.equals("ARGUMENT")) {
                     $arguments_list.add(new JArgument($argument.getStringAttribute("NAME"),
                                                      $argument.getStringAttribute("TYPE"))); 
                  } else if ($current_tag.equals("SETTER")){
                     LinkedList $setter_arg_list = new LinkedList();
                     Enumeration $setter_arguments = $argument.enumerateChildren();
                     while ($setter_arguments.hasMoreElements()) {
                        XMLElement $current = (XMLElement)$setter_arguments.nextElement();
                        if ($current.getName().equals("ARGUMENT")){
                           $setter_arg_list.add(new JArgument($current.getStringAttribute("NAME"),
                                                              $current.getStringAttribute("TYPE")));
                        }
                     }
                     $setters_list.add(new JSetter($argument.getStringAttribute("NAME"),$setter_arg_list));
                  } else {
                     return _dispatcher.error("Bad tag: " + $current_tag +". ARGUMENT expected.");
                  }
               }
               String $alternative_id_property = $alternative.getStringAttribute("ID");
               int $alternative_id = 0;
               if ($alternative_id_property != null) {
                  try {
                     $alternative_id = Integer.parseInt($alternative_id_property);
                  } catch (NumberFormatException e) {
                     throw new JonathanException("Tag ID of alternative at line " + $alternative.getLineNr() +
                                                 " must be a positive integer.");
                  }
               } else {
                  throw new JonathanException("Tag ID of alternative at line " + $alternative.getLineNr() +
                                              " required.");
               }
               $alternatives_list.add(new JAlternative($alternative.getStringAttribute("INIT"),
                                                       $alternative.getStringAttribute("CLASS"),
                                                       $arguments_list,
                                                       $setters_list,
                                                       $alternative_id));
            }
         }

         return new JAtom($class,$alternatives_list);

      } else {
         return _dispatcher.error("XMLAtomDecoder error: Wrong tag " + $current_tag);
      }
   }
}
