/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 *
 * -------------------------------------------------------------------
 * $ID$
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.ContextFactory;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.JonathanException;
import org.objectweb.jonathan.apis.kernel.ClassPathFinder;

import nanoxml.XMLElement;


/**
 * Dispatches XML to runtime component decoding among specific decoders.
 */
public interface XMLElementDecoderDispatcher {

   /**
    * Converts a XML element to its corresponding runtime component.
    * @param _xml_element to be decoded
    * @param _element_name the component's element name
    * @param _configuration where the component will be created
    * @param return the corresponding decoded runtime component
    */
   Component decode(XMLElement _xml_element,String _element_name,Context _configuration) 
      throws JonathanException;
   
   /**
    * Mere error processing.
    */
   Component error(String message) throws JonathanException;

   /**
    * Returns the XML files finder.
    */
   ClassPathFinder getFinder();

   /**
    * Returns the context factory.
    */
   ContextFactory getContextFactory();
}
