/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 * 
 */

package org.objectweb.jonathan.tools.compilers;

import org.objectweb.jonathan.tools.Kcf2java;
import nanoxml.XMLElement;
import nanoxml.XMLParseException;


import java.io.IOException;
import java.io.Writer;

/**
 * XML Alias tag to runtime component java code compiler.
 * Used by {@link Kcf2java} through its <B>getTagCompiler()</B>
 * method to write Alias tags' java code compilation.
 */
public class AliasCompiler implements TagCompiler {
   
   static int nb = 0;

   /**
    * Translates a XML <code>ALIAS</code> element to the java code
    * creating its corresponding alias runtime component.
    * @param _current_component the Alias XML tag to be compiled
    * @param _element_name its element name
    * @param _configuration_var_name its containing configuration
    *         java code variable name
    * @param _writer where to write the translated java code
    * @param _configuration_name the alias's configuration
    *        name XML property
    */
   public String writeComponent(XMLElement _current_component,
                                String _element_name,
                                String _configuration_var_name,
                                Writer _writer,
                                String _configuration_name)
      throws IOException {
      
      String $current_tag = _current_component.getName();
      
      if ($current_tag.equals("ALIAS")){
         String $alias = _current_component.getStringAttribute("NAME");
         if ($alias == null) { // alias without target: illegal
            throw new IOException("At line " + _current_component.getLineNr() + ", alias " + _element_name + " has no target");
         }
         String $var_name = "ALIAS" + (nb++); 
         _writer.write("         Component " + $var_name + 
                       " = new org.objectweb.jonathan.libs.kernel.decoders.JAlias(\"" + 
                       Kcf2java.normalize($alias)+ "\",'/');\n");
         return $var_name;
      } else {
         throw new IOException("TAG error at line " + _current_component.getLineNr() + ":\n" +
                               "ALIAS excepted instead of " + $current_tag);
      }      
   }
}
