/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Priscille Durville
 * 
 */

package org.objectweb.jonathan.tools.compilers;

import org.objectweb.jonathan.tools.Kcf2java;
import nanoxml.XMLElement;
import nanoxml.XMLParseException;
import org.objectweb.jonathan.tools.compilers.TagCompiler;

import java.io.IOException;
import java.io.Writer;

import java.util.Enumeration;

/**
 * XML Assemblage tag to runtime Assemblage component java code compiler.
 * Used by {@link Kcf2java} through its <B>getTagCompiler()</B>
 * method to write Assemblage tags' java code compilation.
 */
public class AssemblageCompiler implements TagCompiler {

   static int nb = 0;
   static int alternatives_list_number = 0;
   static int arguments_list_number = 0;
   static int setters_list_number = 0;
   static int setters_args_number = 0;

   /**
    * Translates a XML <code>ASSEMBLAGE</code> element to the java code
    * creating its corresponding assemblage runtime component.
    * @param _current_component <code>ASSEMBLAGE </code> tag to be decoded
    * @param _element_name the assemblage's element name
    * @param _configuration_var_name the assemblage's configuration's
    * variable name
    * @param _writer where the property assemblage code is written
    * @param _configuration_name the assemblage's configuration's
    * name
    * @param return the corresponding assemblage component code variable
    * name
    */
   public String  writeComponent(XMLElement _current_component,
                                 String _element_name,
                                 String _configuration_var_name,
                                 Writer _writer,
                                 String _configuration_name)
      throws IOException {

      String $current_tag = _current_component.getName();
      
      if ($current_tag.equals("ASSEMBLAGE")){
         String $var_name = "ASSEMBLAGE" + (nb++);
         _element_name = Kcf2java.normalize(_element_name);

         String $factory_name = null;
         String $configuration_name = null;

         Enumeration $members = _current_component.enumerateChildren();
         String $alternatives_list_var = "$s_alternatives_list" + (alternatives_list_number++);
         _writer.write("        java.util.LinkedList " + $alternatives_list_var + 
                       " = new java.util.LinkedList();\n");

         while ($members.hasMoreElements()) {
            XMLElement $argument = (XMLElement) $members.nextElement();
            $current_tag = $argument.getName();
            if ($current_tag.equals("MY_FACTORY")) {
               if ($factory_name != null) {
                  throw new IOException("Duplicate factory declaration at line " + 
                                        _current_component.getLineNr());
               } else {
                  Enumeration $factory_members = $argument.enumerateChildren();
                  while ($factory_members.hasMoreElements()) {
                     XMLElement $factory_member = (XMLElement) $factory_members.nextElement();
                     if ($factory_name != null) {
                        throw new IOException("More than one component in MY_FACTORY at line " + 
                                              $factory_member.getLineNr());
                     } else {
                        $factory_name = Kcf2java.writeComponent($factory_member,"Factory",
                                                                null, _writer,
                                                                _configuration_name + "/" + _element_name);
                     }
                  }
               }
            } else if ($current_tag.equals("MY_CONFIGURATION")) {
               if ($configuration_name != null) {
                  throw new IOException("Duplicate configuration declaration at line " + 
                                        _current_component.getLineNr());
               } else {
                  Enumeration $configuration_members = $argument.enumerateChildren();
                  while ($configuration_members.hasMoreElements()) {
                     XMLElement $configuration_member = (XMLElement) $configuration_members.nextElement();
                     if ($configuration_name != null) {
                        throw new IOException("More than one component in MY_CONFIGURATION at line " + 
                                              $configuration_member.getLineNr());
                     } else {
                        $configuration_name = 
                           Kcf2java.writeComponent($configuration_member,"Configuration",
                                                     null, _writer,
                                                     _configuration_name + "/" + _element_name);
                     }
                  }
               }

            } else if ($current_tag.equals("ALTERNATIVE")) {
               String $arguments_list_var = "$s_arguments_list" + (arguments_list_number++);
               _writer.write("        java.util.LinkedList " + $arguments_list_var + 
                             " = new java.util.LinkedList();\n");
               String $setters_list_var = "$s_setters_list" + (setters_list_number++);
               _writer.write("        java.util.LinkedList " + $setters_list_var + 
                             " = new java.util.LinkedList();\n");
               Enumeration $arguments = $argument.enumerateChildren();
               while ($arguments.hasMoreElements()) {
                  XMLElement $arg = (XMLElement) $arguments.nextElement();
                  $current_tag = $arg.getName();
                  if ($current_tag.equals("ARGUMENT")) {
                     _writer.write("         " + $arguments_list_var + 
                                   ".add(new org.objectweb.jonathan.libs.kernel.decoders.JArgument(\"" +
                                   Kcf2java.normalize($arg.getStringAttribute("NAME")) + "\",\"" +
                                   Kcf2java.normalize($arg.getStringAttribute("TYPE")) + "\"));\n"); 
                  } else if ($current_tag.equals("SETTER")){
                     String $args_list_var = "$s_args_list" + (setters_args_number++);
                     _writer.write("        java.util.LinkedList " + $args_list_var + 
                             " = new java.util.LinkedList();\n");
                     Enumeration $args = $arg.enumerateChildren();
                     while ($args.hasMoreElements()) {
                        XMLElement $current = (XMLElement) $args.nextElement();
                        String $tag = $current.getName();
                        if ($tag.equals("ARGUMENT")) {
                            _writer.write("         " + $args_list_var + 
                                          ".add(new org.objectweb.jonathan.libs.kernel.decoders.JArgument(\"" +
                                          Kcf2java.normalize($current.getStringAttribute("NAME")) + "\",\"" +
                                          Kcf2java.normalize($current.getStringAttribute("TYPE")) + "\"));\n"); 
                        }
                     }
                     _writer.write("         " + $setters_list_var + 
                                   ".add(new org.objectweb.jonathan.libs.kernel.decoders.JSetter(\"" +
                                   Kcf2java.normalize($arg.getStringAttribute("NAME")) + "\"," +
                                   $args_list_var + "));\n"); 
                  } else {
                     throw new IOException("Bad tag: " + $current_tag +". ARGUMENT expected.");
                  }
               }
               _writer.write("         " + $alternatives_list_var + 
                             ".add(new  org.objectweb.jonathan.libs.kernel.decoders.JAlternative(\"" +
                             Kcf2java.normalize($argument.getStringAttribute("INIT")) + "\"," + "\"" +
                             Kcf2java.normalize($argument.getStringAttribute("CLASS")) + "\"," +
                             $arguments_list_var + "," + 
                             $setters_list_var + "," + 
                             $argument.getStringAttribute("ID") + "));\n");
            } else if ($current_tag.equals("DOCUMENTATION")){
               //no exception.
            } else {
               throw new IOException("Bad tag: " + $current_tag +". ALTERNATIVE expected.");
            }
         } 

         String $factory_attribute = _current_component.getStringAttribute("FACTORY");
         if ($factory_attribute != null) {
            $factory_attribute = Kcf2java.normalize($factory_attribute);
            if (! $factory_attribute.startsWith("/")) {
               $factory_attribute = "../" + $factory_attribute;
            }
         }
         if ($factory_attribute == null && $factory_name == null) {
            throw new IOException("No factory defined for assemblage at line: " + 
                                     _current_component.getLineNr());
         } else if ($factory_attribute != null && $factory_name != null) {            
            throw new IOException("Duplicate configuration declaration at line " + 
                                     _current_component.getLineNr());
         } else if ($factory_attribute != null) {
            $factory_name = $var_name + "_factory";
            _writer.write("         Component " + $factory_name +
                          " = new org.objectweb.jonathan.libs.kernel.decoders.JAlias(\"" + 
                          Kcf2java.normalize($factory_attribute) + "\",'/');\n");
         }
         
         String $configuration_attribute = _current_component.getStringAttribute("CONFIGURATION");
         if ($configuration_attribute != null) {
            $configuration_attribute = Kcf2java.normalize($configuration_attribute);
            if (! $configuration_attribute.startsWith("/")) {
               $configuration_attribute = "../" + $configuration_attribute;
            }
         }
         if ($configuration_attribute == null && $configuration_name == null) {
            //throw new IOException("No configuration defined for assemblage at line: " + _current_component.getLineNr());
            XMLElement $conf = new XMLElement();
            $conf.setName("CONFIGURATION");
            $configuration_name = Kcf2java.writeComponent($conf,"Configuration",null, _writer,_configuration_name + "/" + _element_name);
         } else if ($configuration_attribute != null && $configuration_name != null) {            
            throw new IOException("Duplicate configuration declaration at line " + 
                                     _current_component.getLineNr());
         } else if ($configuration_attribute != null) {
            $configuration_name = $var_name + "_configuration";
            _writer.write("         Component " + $configuration_name +
                          " = new org.objectweb.jonathan.libs.kernel.decoders.JAlias(\"" + 
                          Kcf2java.normalize($configuration_attribute) + "\",'/');\n");
         }

         String $alternative = _current_component.getStringAttribute("ALTERNATIVE");
         if ($alternative != null) {
            try {
               int $alt = Integer.parseInt($alternative);
               if ($alt < 0) {
                  throw new IOException("Alternative must be specified as a POSITIVE integer at line " +
                                        _current_component.getLineNr());
               }
            } catch (NumberFormatException e) {
               throw new IOException("Alternative must be specified as an integer at line " +
                                     _current_component.getLineNr());
            }
         }
         if ($alternative != null) {
            _writer.write("         Component " + $var_name + 
                          " = new org.objectweb.jonathan.libs.kernel.decoders.JAssemblage(" + 
                          $factory_name + "," + $configuration_name + "," +
                          $alternative + "," + $alternatives_list_var + ");\n");
         } else {
            _writer.write("         Component " + $var_name + 
                          " = new org.objectweb.jonathan.libs.kernel.decoders.JAssemblage(" + 
                          $factory_name + "," + $configuration_name + "," +
                          $alternatives_list_var + ");\n");
         }
         return $var_name;
      } else {
         throw new IOException("TAG error at line " + _current_component.getLineNr() + ":\n" +
                               "ASSEMBLAGE excepted instead of " + $current_tag);
      }
   }
}
