/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Priscille Durville
 * 
 */

package org.objectweb.jonathan.tools.compilers;

import org.objectweb.jonathan.tools.Kcf2java;
import nanoxml.XMLElement;
import nanoxml.XMLParseException;
import org.objectweb.jonathan.tools.compilers.TagCompiler;

import java.io.IOException;
import java.io.Writer;

import java.util.Enumeration;

/**
 * XML Atom tag to runtime Atom component java code compiler.
 * Used by {@link Kcf2java} through its <B>getTagCompiler()</B>
 * method to write Atom tags' java code compilation.
 */
public class AtomCompiler implements TagCompiler {

   static int nb = 0;
   static int alternatives_list_number = 0;
   static int arguments_list_number = 0;
   static int setters_list_number = 0;
   static int setters_args_number = 0;

   /**
    * Translates a XML <code>ATOM</code> element to the java code
    * creating its corresponding atom runtime component.
    * @param _current_component the Atom XML tag to be compiled
    * @param _element_name its element name
    * @param _configuration_var_name its containing configuration
    * java code variable name
    * @param _writer where to write the translated java code
    * @param _configuration_name the atom's configuration
    * name XML property
    */
   public String  writeComponent(XMLElement _current_component,
                                 String _element_name,
                                 String _configuration_var_name,
                                 Writer _writer,
                                 String _configuration_name)
      throws IOException {
      
      String $current_tag = _current_component.getName();
      
      if ($current_tag.equals("ATOM")){
         String $var_name = "ATOM" + (nb++);
         String $class = _current_component.getStringAttribute("CLASS");

         Enumeration $alternatives = _current_component.enumerateChildren();
         String $alternatives_list_var = "$a_alternatives_list" + (alternatives_list_number++);
         _writer.write("        java.util.LinkedList " + $alternatives_list_var + 
                       " = new java.util.LinkedList();\n");
         while ($alternatives.hasMoreElements()) {
            XMLElement $alternative = (XMLElement) $alternatives.nextElement();
            $current_tag = $alternative.getName();
            if ($current_tag.equals("ALTERNATIVE")) {
               String $arguments_list_var = "$a_arguments_list" + (arguments_list_number++);
               _writer.write("        java.util.LinkedList " + $arguments_list_var + 
                             " = new java.util.LinkedList();\n");
               String $setters_list_var = "$a_setters_list" + (setters_list_number++);
               _writer.write("        java.util.LinkedList " + $setters_list_var + 
                             " = new java.util.LinkedList();\n");
               Enumeration $arguments = $alternative.enumerateChildren();
               while ($arguments.hasMoreElements()) {
                  XMLElement $argument = (XMLElement) $arguments.nextElement();
                  $current_tag = $argument.getName();
                  if ($current_tag.equals("ARGUMENT")) {
                     _writer.write("         " + $arguments_list_var + 
                                   ".add(new org.objectweb.jonathan.libs.kernel.decoders.JArgument(\"" +
                                   Kcf2java.normalize($argument.getStringAttribute("NAME")) + "\",\"" +
                                   Kcf2java.normalize($argument.getStringAttribute("TYPE")) + "\"));\n"); 
                  } else if ($current_tag.equals("SETTER")){
                     String $args_list_var = "$a_args_list" + (setters_args_number++);
                     _writer.write("        java.util.LinkedList " + $args_list_var + 
                             " = new java.util.LinkedList();\n");
                     Enumeration $args = $argument.enumerateChildren();
                     while ($args.hasMoreElements()) {
                        XMLElement $current = (XMLElement) $args.nextElement();
                        String $tag = $current.getName();
                        if ($tag.equals("ARGUMENT")) {
                            _writer.write("         " + $args_list_var + 
                                          ".add(new org.objectweb.jonathan.libs.kernel.decoders.JArgument(\"" +
                                          Kcf2java.normalize($current.getStringAttribute("NAME")) + "\",\"" +
                                          Kcf2java.normalize($current.getStringAttribute("TYPE")) + "\"));\n"); 
                        }
                     }
                     _writer.write("         " + $setters_list_var + 
                                   ".add(new org.objectweb.jonathan.libs.kernel.decoders.JSetter(\"" +
                                   Kcf2java.normalize($argument.getStringAttribute("NAME")) + "\"," +
                                   $args_list_var + "));\n"); 
                  } else {
                     throw new IOException("Bad tag: " + $current_tag +". ARGUMENT expected.");
                  }
               }
               _writer.write("         " + $alternatives_list_var + 
                             ".add(new org.objectweb.jonathan.libs.kernel.decoders.JAlternative(\"" +
                             Kcf2java.normalize($alternative.getStringAttribute("INIT")) + "\"," + "\"" +
                             Kcf2java.normalize($alternative.getStringAttribute("CLASS")) + "\"," +
                             $arguments_list_var + "," + 
                             $setters_list_var + "," + 
                             $alternative.getStringAttribute("ID") + "));\n");
            } else if ($current_tag.equals("DOCUMENTATION")){
               //no exception.
            } else {
               throw new IOException("Bad tag: " + $current_tag +". ALTERNATIVE expected.");
            }
         }

         _writer.write("         Component " + $var_name + 
                       " = new org.objectweb.jonathan.libs.kernel.decoders.JAtom(\"" +
                       Kcf2java.normalize($class) + "\"," +
                       $alternatives_list_var + ");\n");
         return $var_name;
      } else {
         throw new IOException("TAG error at line " + _current_component.getLineNr() + ":\n" +
                               "ATOM excepted instead of " + $current_tag);
      }
   }
}
