/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 * 
 */

package org.objectweb.jonathan.tools.compilers;

import nanoxml.XMLElement;
import nanoxml.XMLParseException;


import java.io.IOException;
import java.io.Writer;

/**
 * XML tag to runtime component java code compiler interface.
 * Used by {@link org.objectweb.jonathan.tools.Kcf2java} through its <B>getTagCompiler()</B>
 * method to dispatch component tags to java code compilation.
 * Has to be implemented for each runtime component specific type.
 * The configuration tag compiler is not needed, since Kcf2java
 * has its mechanism as a built-in one.
 */
public interface TagCompiler {
   
   /**
    * Translates a XML element to the java code
    * creating its corresponding runtime component(s).
    * @param _current_component the XML tag to be compiled
    * @param _element_name its element name
    * @param _configuration_var_name its containing configuration
    *                                Java code variable name
    * @param _writer where to write the translated java code
    * @param _configuration_name the configuration absolute name
    * @return the name of the variable holding the component in the Java code
    * @exception IOException if something goes wrong.
    */
   String writeComponent(XMLElement _current_component,
                         String _element_name,
                         String _configuration_var_name,
                         Writer _writer,
                         String _configuration_name)
      throws IOException;
}
